//------------------------------------------------------------------------------
//	$Id: node-modifier.js,v 1.80 2021/07/14 20:36:56 wolf Exp $
//------------------------------------------------------------------------------
//	See:	https://www.netzwolf.info/code/addons/genderclown/
//------------------------------------------------------------------------------
//	License: WTF public license (see: http://www.wtfpl.net/)
//------------------------------------------------------------------------------

/*jsl:option explicit*/

'use strict';

//------------------------------------------------------------------------------
//
//	NodeModifier: Object defined after executing initialization code
//
//------------------------------------------------------------------------------

var NodeModifier = (function( obj ) {

	var startTime = new Date().getTime();
	console.debug( 'node-modifier.js started @' + new Date().toTimeString() );
	obj.init();
	console.debug( 'node-modifier.js initialization: ' +
			( new Date().getTime() - startTime ) + ' ms' );
	return obj;
}({

//------------------------------------------------------------------------------
//
//	Common regular expressions
//
//------------------------------------------------------------------------------

RegExpNoLetters: new RegExp( '\\P{L}+', 'gu' ),

//==============================================================================
//
//	B I N N E N - I
//
//==============================================================================

//------------------------------------------------------------------------------
//
//
//	Pattern:	~In
//
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	Case sensitive "~In"-Pattern
//------------------------------------------------------------------------------

RegExpCInCS: new RegExp(
	'(' +
		'(?:' + '\\p{L}+' + '\xad' + ')*' + '\\p{L}*' + '\\p{Ll}' +
	')' +
	'(' +
		'In' +
	')' +
	'', 'u' ),

RegExpInCS: new RegExp(
	// word boundary
	'(?<!\\p{L})' +
	'(' +
		'(?:' + '\\p{L}+' + '\xad' + ')*' + '\\p{L}*' + '\\p{Ll}' +
	')' +
	'(' +
		'In' +
	')' +
	// word boundary; -in- or /in/ in URLs
	'(?!' +
		'[\\p{L}\\p{N}\xad(/-]' +
	')' +
	'', 'ug' ),

//------------------------------------------------------------------------------
//	Case insensitive "~*In"-Pattern
//------------------------------------------------------------------------------

RegExpCInCI: new RegExp(
	// word prefix
	'(' +
		'(?:' + '\\p{L}+' + '\xad' + ')*' + '\\p{L}+' +
	')' +
	// Chef*s*in
	'(?:' +
		'[' + '\\p{Pc}\\p{Pd}\\p{Po}' + ']' + 's' +
	')?' +
	// (in), /-in, *in, -in, :in, ïn
	'(' +
		'\\p{Ps}' + 'in' + '\\p{Pe}' +
	'|' +
		'(?:' +
			'/-' +
		'|' +
			'(?!' + '[,]' + ')' +
			'[' + '\\p{Pc}\\p{Pd}\\p{Po}\\p{Sm}' + ']' +
		')' +
		'in' + '(?!\xad)' +
	'|' +
		'ïn' +
	')' +
	'' , 'ui' ),

RegExpInCI: new RegExp(
	// word boundary
	'(?<!' +
		'\\p{L}' +
	')' +
	// not element of file path
	'(?<!' +
		'^' + '/?' +
		'(?:' +
			'\\w+' + '[-/]' +
		')+' +
	')' +
	// not element of url
	'(?<!' +
		'(?:' + '[a-z]+://' + '|' + '@' + ')' +
		'(?:' +
			'[\\w%]+' + '[./-]' +
		'){2,}' +
	')' +
	// word prefix
	'(' +
		'(?:' + '\\p{L}+' + '\xad' + ')*' + '\\p{L}+' +
	')' +
	// Chef*s*in
	'(?:' +
		'[' + '\\p{Pc}\\p{Pd}\\p{Po}' + ']' + 's' +
	')?' +
	// (in), /-in, *in, -in, :in, ïn
	'(' +
		'\\p{Ps}' + 'in' + '\\p{Pe}' +
	'|' +
		'(?:' +
			'/-' +
		'|' +
			'(?!' + '[,]' + ')' +
			'[' + '\\p{Pc}\\p{Pd}\\p{Po}\\p{Sm}' + ']' +
		')' +
		'in' + '(?!\xad)' +
	'|' +
		'ïn' +
	')' +
	// word boundary; "(" may be function call; -in- or /in/ in URLs
	'(?!' +
		'[\\p{L}\\p{N}\xad(/-]' +
	')' +
	'', 'ugi' ),

//------------------------------------------------------------------------------
//	Cleaning pattern and lookup table
//------------------------------------------------------------------------------

MappingInCleaning: null,
RegExpInCleaning:   null,
 
//------------------------------------------------------------------------------
//	Generate cleaning pattern and lookup table from transformation table
//------------------------------------------------------------------------------

tableInCleaning: [

	{ b: 'Anwält',	r: 'Anwalt'	},
	{ b: 'Arzt',	r: 'Arzt'	},
	{ b: 'Ärzt',	r: 'Arzt'	},
	{ b: 'Bäuer',	r: 'Bauer'	},
	{ b: 'Bäur',	r: 'Bauer'	},
	{ b: 'Gräf',	r: 'Graf'	},
	{ b: 'Gäst',	r: 'Gast'	},
	{ b: 'Gött',	r: 'Gott'	},
	{ b: 'Häs',	r: 'Hase'	},
	{ b: 'Hünd',	r: 'Hund'	},
	{ b: 'Jüd',	r: 'Jude'	},
	{ b: 'Kund',	s: 'e'		},
	{ b: 'Köch',	r: 'Koch'	},
	{ b: 'Rät',	r: 'Rat'	},
	{ b: 'Schwäb',	r: 'Schwabe'	},
	{ b: 'b',	s: 'e'		},	// Schwab-e
	{ b: 'ieb',	s: ''		},	// Dieb
	{ b: 'c',	s: ''		},
	{ b: 'd',	s: ''		},	// Doktorand, Feind, Freund
	{ b: 'er',	s: ''		},
	{ b: 'f',	s: ''		},	// Chef
	{ b: 'g',	s: 'e'		},	// Kolleg-e, Psycholog-e
	{ b: 'ig',	s: ''		},	// König
	{ b: 'ch',	s: 'e'		},	// Wach-e
	{ b: 'sch',	s: ''		},	// Mensch
	{ b: 'k',	s: 'e'		},
	{ b: 'l',	s: ''		},
	{ b: 'm',	s: ''		},	// Muslim
	{ b: 'n',	s: ''		},	// Fan
	{ b: 'p',	s: ''		},
	{ b: 'r',	s: ''		},	// Arbeiter, Archivar, Autor, Doktor, Gegner, Installateur, Kommissar, Partner, Professor
	{ b: 's',	s: 'e'		},
	{ b: 't',	s: ''		},	// Architekt, Dozent, Jurist, Soldat, Student, Wirt
	{ b: 'amt',	s: 'er'		},	// Beamt-er
	{ b: 'Brit',	s: 'e'		},	// Brit-e
	{ b: 'rt',	s: 'e'		},	// Expert-e
	{ b: 'irt',	s: ''		},	// Wirt
	{ b: 'st',	s: ''		},	// Aktivist, Jurist
	{ b: 'v',	s: 'e'		},
	{ b: 'w',	s: 'e'		},
	{ b: 'z',	s: 'e'		}
],

prepareInCleaning: function() {

	//----------------------------------------------------------------------
	//	BeamtIn → Beamter
	//----------------------------------------------------------------------

	var wordList = [];
	var mappings = {};

	for( var i = 0 ; i < this.tableInCleaning.length ; ++i ) {

		var entry = this.tableInCleaning[ i ];

		//--------------------------------------------------------------
		//	pattern → pattern + suffix
		//	pattern → replacement
		//--------------------------------------------------------------

		var tc_b = entry.b;
		var lc_b = tc_b.toLowerCase();
		var uc_b = tc_b.toUpperCase();

		wordList.push( tc_b );
		wordList.push( lc_b );
		wordList.push( uc_b );

		if( entry.r ) {

			mappings[ tc_b ] = entry.r;
			mappings[ lc_b ] = entry.r.toLowerCase();
			mappings[ uc_b ] = entry.r.toUpperCase();

		} else if( entry.s !== undefined ) {

			mappings[ tc_b ] = tc_b + entry.s;
			mappings[ lc_b ] = lc_b + entry.s;
			mappings[ uc_b ] = uc_b + entry.s.toUpperCase();

		}
	}

	//----------------------------------------------------------------------
	//	create regexp and lookup table
	//----------------------------------------------------------------------

	var regex = '^(.*?)(' +
			wordList.sort( this.byLength ).join( '|' ) +
			')$';

	this.RegExpInCleaning = new RegExp( regex );
	this.MappingInCleaning = mappings;
},

//------------------------------------------------------------------------------
//
//
//	Pattern:	~Innen
//
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	Case sensitive "~Innen"-Pattern
//------------------------------------------------------------------------------

RegExpCInnenCS: new RegExp(
	// word
	'(' +
		'(?:' + '\\p{L}+' + '\xad' + ')*' + '\\p{L}*' + '\\p{Ll}' +
	')' +
	// innen
	'(' +
		'In' + '\xad?' + 'nen' +
	'|' +
		'IN' + '\xad?' + 'N[Ee][Nn]' +
	')' +
	'', 'u' ),
 
RegExpInnenCS: new RegExp(
	// word boundary
	'(?<!\\p{L})' +
	// word
	'(' +
		'(?:' + '\\p{L}+' + '\xad' + ')*' + '\\p{L}*' + '\\p{Ll}' +
	')' +
	// Innen / INNEN
	'(' +
		'In' + '\xad?' + 'nen' +
	'|' +
		'IN' + '\xad?' + 'N[Ee][Nn]' +
	')' +
	// compound
	'(' +
		'(?:' + '\xad?' + '\\p{Ll}+' + ')*' +
	')' +
	// word boundary
	'(?!' +
		'[\\p{L}\\p{N}\xad(]' +
	')' +
	'', 'ug' ),
 
//------------------------------------------------------------------------------
//	Case insensitive "~*Innen"-Pattern
//------------------------------------------------------------------------------

RegExpCInnenCI: new RegExp(
	// word
	'(' +
		'(?:' + '\\p{L}+' + '\xad' + ')*' + '\\p{L}+' +
	')' +
	// innen
	'(' +			// Student(innen)
		'\\p{Ps}' + 'in' + '\xad?' + 'nen' + '\\p{Pe}' +
	'|' +			// Student(inn)en
		'\\p{Ps}' + 'inn' + '\\p{Pe}' + 'en' +
	'|' +
		'(?:' +			// Student/-innen
			'/-' +
		'|' +			// Student:innen
			'[' + '\\p{Pc}\\p{Pd}\\p{Po}\\p{Sm}' + ']' +
		')' +
		'in' + '\xad?' + 'n' + '\\p{P}?' + 'e' + '\\p{P}?' + 'n' +
	'|' +
		'ïn' + '\xad?' + 'nen' +
	'|' +
		'\\s+' + '(?:und|u[.]|oder|o[.])' + '\\s+' + '-innen' +
	')' +
	'', 'ui' ),

RegExpInnenCI: new RegExp(
	// word boundary
	'(?<!\\p{L})' +
	// not element of url
	'(?<!' +
		'\\w+://' +
		'(?:' +
			'[\\w%]+' + '[./-]' +
		'){2,}' +
	')' +
	// word
	'(' +
		'(?:' + '\\p{L}+' + '\xad' + ')*' + '\\p{L}+' +
	')' +
	// innen
	'(' +			// Student(innen)
		'\\p{Ps}' + 'in' + '\xad?' + 'nen' + '\\p{Pe}' +
	'|' +			// Student(inn)en
		'\\p{Ps}' + 'inn' + '\\p{Pe}' + 'en' +
	'|' +
		'(?:' +			// Student/-innen
			'/-' +
		'|' +			// Student:innen
			'[' + '\\p{Pc}\\p{Pd}\\p{Po}\\p{Sm}' + ']' +
		')' +
		'in' + '\xad?' + 'n' + '\\p{P}?' + 'e' + '\\p{P}?' + 'n' +
	'|' +
		'ïn' + '\xad?' + 'nen' +
	'|' +
		'\\s+' + '(?:und|u[.]|oder|o[.])' + '\\s+' + '-innen' +
	')' +
	// compound
	'(' +
		'(?:' + '\xad?' + '\\p{Ll}+' + ')*' +
	')' +
	// word boundary
	'(?!' +
		'[\\p{L}\\p{N}\xad(]' +
	')' +
	'', 'ugi' ),

//------------------------------------------------------------------------------
//	Cleaning pattern and lookup table
//------------------------------------------------------------------------------

MappingInnenCleaning: null,
RegExpInnenCleaning:   null,

//------------------------------------------------------------------------------
//	Generate cleaning pattern and lookup table from transformation table
//------------------------------------------------------------------------------

tableInnenCleaning: [

	{ b: 'Bäuer',		r: 'Bauern' },
	{ b: 'Bäur',		r: 'Bauern' },
	{ b: 'Chef',		s: 's'	},
	{ b: 'Doktor',		s: 'es'	},
	{ b: 'Fan',		s: 's'	},
	{ b: 'Gäst',		s: 'e'	},
	{ b: 'Gräf',		r: 'Grafen' },
	{ b: 'Jüd',		r: 'Juden' },
	{ b: 'Schwäb',		r: 'Schwaben' },
	{ b: 'b',		s: 'en'	},
	{ b: 'ieb',		s: 'e'	},
	{ b: 'd',		s: 'e'	},	// Feind-e
	{ b: 'and',		s: 'en'	},	// Doktorand-en
	{ b: 'und',		s: 'en'	},	// Kund-en
	{ b: 'eund',		s: 'e'	},	// Freund-e
	{ b: 'e',		s: ''	},
	{ b: 'f',		s: 'en'	},	// Graf-en
	{ b: 'g',		s: 'e'	},	// König-e
	{ b: 'eg',		s: 'en'	},	// Kolleg-en
	{ b: 'log',		s: 'en'	},	// Psycholog-en
	{ b: 'eug',		s: 'en'	},	// Zeug-en
	{ b: 'h',		s: 'en'	},
	{ b: 'ch',		s: 'e'	},	// Köch-e
	{ b: 'k',		s: 'en'	},
	{ b: 'l',		s: 'e'	},
	{ b: 'el',		s: ''	},
	{ b: 'm',		s: 'e'	},	// Muslim-e
	{ b: 'n',		s: 'en'	},	// Grün-en
	{ b: 'p',		s: 'en'	},
	{ b: 'r',		s: 'e'	},	// Archivar-e, Installateur-e, Kommissar-e
	{ b: 'bar',		s: 'n'	},	// Nachbar-n
	{ b: 'gar',		s: 'n'	},	// Ungar-n
	{ b: 'er',		s: ''	},	// Arbeiter, Partner
	{ b: 'ier',		s: 'e'	},	// Polier-e
	{ b: 'rier',		s: ''	},	// Parlamentari-er
	{ b: 'or',		s: 'en'	},	// Autor-en, Professor-en
	{ b: 's',		s: 'en'	},	// Belaruss-en
	{ b: 't',		s: 'en'	},	// Architekt-en, Dozent-en, Expert-en, Jurist-en, Soldat-en, Student-en
	{ b: 'ät',		s: 'e'	},	// Rät-e
	{ b: 'st',		s: 'en'	},	// Aktivist-en
	{ b: 'äst',		s: 'e'	},	// Gäst-e
	{ b: 'it',		s: 'en'	},
	{ b: 'brit',		s: 'en'	},	// Brit-en
	{ b: 'lt',		s: 'e'	},	// Anwält-e
	{ b: 'rt',		s: 'e'	},	// Wirt-e
	{ b: 'ert',		s: 'en'	},	// Expert-en
	{ b: 'mt',		s: 'e'	},	// Beamt-e
	{ b: 'tt',		s: 'er'	},	// Gött-er
	{ b: 'zt',		s: 'e'	},	// Ärzt-e
	{ b: 'v',		s: 'en'	},
	{ b: 'w',		s: 'en'	}
],

prepareInnenCleaning: function() {

	//----------------------------------------------------------------------
	//	BeamtInnen → Beamte
	//----------------------------------------------------------------------

	var wordList = [];
	var mappings = {};

	for( var i = 0 ; i < this.tableInnenCleaning.length ; ++i ) {

		var entry = this.tableInnenCleaning[ i ];

		//--------------------------------------------------------------
		//	pattern → pattern + suffix
		//	pattern → replacement
		//--------------------------------------------------------------

		var tc_b = entry.b;
		var lc_b = tc_b.toLowerCase();
		var uc_b = tc_b.toUpperCase();

		wordList.push( tc_b );
		wordList.push( lc_b );
		wordList.push( uc_b );

		if( entry.r ) {

			mappings[ tc_b ] = entry.r;
			mappings[ lc_b ] = entry.r.toLowerCase();
			mappings[ uc_b ] = entry.r.toUpperCase();

		} else if( entry.s !== undefined ) {

			mappings[ tc_b ] = tc_b + entry.s;
			mappings[ lc_b ] = lc_b + entry.s;
			mappings[ uc_b ] = uc_b + entry.s.toUpperCase();
		}
	}

	//----------------------------------------------------------------------
	//	create regexp and lookup table
	//----------------------------------------------------------------------

	var regex = '^(.*?)(' +
			wordList.sort( this.byLength ).join( '|' ) +
			')$';

	this.RegExpInnenCleaning = new RegExp( regex );
	this.MappingInnenCleaning = mappings;
},

//==============================================================================
//
//	P R E S E N T   P A R T I C I P L E
//
//==============================================================================

//------------------------------------------------------------------------------
//
//
//	Pattern:	Studierende → Studenten, Arbeitende → Arbeiter
//
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	Cleaning pattern and lookup table
//------------------------------------------------------------------------------

RegExpParticiple: null,
RegExpCParticiple: null,
MappingParticiple: null,

//------------------------------------------------------------------------------
//	Generate cleaning pattern and lookup table from transformation table
//------------------------------------------------------------------------------

tableParticipleCleaning: [

	{ w: 'Assistent'      },
	{ w: 'Demonstrant'    },
	{ w: 'Diener'         },
	{ w: 'Dozent'         },
	{ w: 'Student'        },
	{ w: 'Arbeiter'       },
	{ w: 'Fahrer'         },
	{ w: 'Fahrrad Fahrer' , r: 'Radfahrer' },
	{ w: 'Forscher'       },
	{ w: 'Geber'          },
	{ w: 'Kaufer'         , r: 'Käufer'    },
	{ w: 'Kunstschaffer'  , r: 'Künstler'  },
	{ w: 'Musikschaffer'  , r: 'Musiker'   },
	//{ w: 'Laufer'         , r: 'Läufer'    }, // auf dem Laufenden
	{ w: 'Lehrer'         },
	{ w: 'Lerner'         , r: 'Schüler'   },
	{ w: 'Leser'          },
	{ w: 'Nehmer'         },
	{ w: 'Nutzer'         },
	{ w: 'Polizeier'      , r: 'Polizisten'},
	{ w: 'Repräsentant'   },
	{ w: 'Schreiber'      },
	{ w: 'Wohner'         },
	{ w: 'Zu Fuß Geher'   , r: 'Fußgänger' }
],

prepareParticipleCleaning: function() {

	//----------------------------------------------------------------------
	//	Assistierender → Assistent, Arbeitender → Arbeiter
	//----------------------------------------------------------------------

	var wordList = [];
	var mappings = {};

	for( var i = 0 ; i < this.tableParticipleCleaning.length ; ++i ) {

		var entry = this.tableParticipleCleaning[ i ];

		var word_tc = entry.w;
		var word_lc = word_tc.toLowerCase();

		//--------------------------------------------------------------
		//	Assist|ent →	Assistist|ierend|er	→ Assist-ent
		//--------------------------------------------------------------

		if( word_tc.substr( -2 ) == 'nt' ) {

			var ext = word_tc.substr( -3 );

			var root_tc = word_tc.substr( 0, word_tc.length-3 );
			var root_lc = word_lc.substr( 0, word_tc.length-3 ); 

			wordList.push( root_tc.substr( 1 ) );

			mappings[ root_tc + 'ierend' ] = root_tc + ext;
			mappings[ root_lc + 'ierend' ] = root_lc + ext;

			//------------------------------------------------------
			//	Studierender oder Studierende
			//------------------------------------------------------

			wordList.push(
				root_tc.substr( 1 ) +	// [S]tud
				'ierender' +		// ierender
				'\x20oder\x20' +	// _oder_
				root_tc );		// Stud

			mappings[
				root_tc +		// Stud
				'ierender' +		// ierender
				'\x20oder\x20' +	// _oder_
				root_tc +		// Stud
				'ierend' ] =		// ierend
				root_tc + ext;		// → Student/Repräsentant

		//--------------------------------------------------------------
		//	Arbeit|er →	Arbeit|end|er		→ Arbeit|er
		//--------------------------------------------------------------

		} else if( word_tc.substr( -2 ) == 'er' ) {

			root_tc = word_tc.substr( 0, word_tc.length-2 );
			root_lc = word_lc.substr( 0, word_tc.length-2 ); 

			var repl_tc = entry.r || root_tc + 'er';

			wordList.push( root_tc.substr( 1 ) );

			mappings[ root_tc + 'end' ] = repl_tc;
			mappings[ root_lc + 'end' ] = repl_tc.toLowerCase();
		}
	}

	//----------------------------------------------------------------------
	//	create regexp and lookup table
	//----------------------------------------------------------------------

	this.RegExpCParticiple = new RegExp(
	'(' +
		'\\p{Lu}' + '\\p{L}*' +
	')' +
	'(' +
		wordList.sort().reverse().join( '|' ) +
	')' +
	'(' +
		'end|ierend' +
	')' +
	'(' +
		'er|en|e/r|e' +
	')' +
	'', 'u' );

	this.RegExpParticiple = new RegExp(
	// word boundary
	'(?<!' +
		'\\p{L}' +
	')' +
	'(?<!' +
		'ins\\x20' +
	')' +
	'(' +
		'\\p{Lu}' + '\\p{L}*' +
	')' +
	'(' +
		wordList.sort().reverse().join( '|' ) +
	')' +
	'(' +
		'end|ierend' +
	')' +
	'(' +
		'er|en|e/r|e' +
	')' +
	// word boundary
	'(?!' +
		'\\p{L}' +
	')' +
	'', 'gu' );

	this.MappingParticiple = mappings;
},

//==============================================================================
//
//	D U P L I C A T E   F O R M S
//
//==============================================================================

//------------------------------------------------------------------------------
//
//
//	Pattern:	Beamtinnen und Beamte, Beamte und Beamtinnen
//
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	Cleaning pattern
//------------------------------------------------------------------------------

RegExpCDuplicates: new RegExp(
	'(?:' +
		'(' +					// #1 = backref
			'\\p{L}+' +			// "Beamt"
		')' +
		'innen' +				// "innen"
		'(?:' +
							// und | u. | oder | o.
			'\\s+'+ '(?:und|u[.]|oder|o[.])' + '\\s+'+
		'|' +
			'[_*/]' +			// Xinnen_Xe
		')' +
		'(' +					// #2 = Beamte
			'\\1' +				// "Beamt"
			'\\p{L}*' +			// "e"
		')' +
	'|' +
		'(' +					// #3 = Beamte
			'(' +				// #4 = backref
				'\\p{L}+' +		// "Beamt"
			')' +
			'\\p{L}*' +			// "e"
		')' +
		'(?:' +
							// und | u. | oder | o.
			'\\s+'+ '(?:und|u[.]|oder|o[.])' +'\\s+'+
		'|' +
			'[_*/]' +			// Xe_Xinnen
		')' +
		'(?:' +
			'\\4' +				// "Beamt"
		')' +
		'innen' +				// "innen"
	')' +
	'', 'iu' ),

RegExpDuplicates: new RegExp(
	// word boundary
	'(?<!' + '\\p{L}' + ')' +
	'(?:' +
		'(' +					// #1 = backref
			'\\p{L}+' +			// "Beamt"
		')' +
		'innen' +				// "innen"
		'(?:' +
							// und | u. | oder | o.
			'\\s+'+ '(?:und|u[.]|oder|o[.])' +'\\s+'+
		'|' +
			'[_*/]' +			// Xinnen_Xe
		')' +
		'(' +					// #2 = Beamte
			'\\1' +				// "Beamt"
			'\\p{L}*' +			// "e"
		')' +
	'|' +
		'(' +					// #3 = Beamte
			'(' +				// #4 = backref
				'\\p{L}+' +		// "Beamt"
			')' +
			'\\p{L}*' +			// "e"
		')' +
		'(?:' +
							// und | u. | oder | o.
			'\\s+'+ '(?:und|u[.]|oder|o[.])' +'\\s+'+
		'|' +
			'[_*/]' +			// Xe_Xinnen
		')' +
		'(?:' +
			'\\4' +				// "Beamt"
		')' +
		'innen' +				// "innen"
	')' +
	// word boundary
	'(?!' + '\\p{L}' + ')' +
	'', 'giu' ),

//------------------------------------------------------------------------------
//
//
//	Pattern:	Deine bzw. Dein	→ Dein
//			Deine oder Dein	→ Dein
//			Deine  /   Dein	→ Dein
//
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	Cleaning pattern and lookup table
//------------------------------------------------------------------------------

RegExpPairs: null,
RegExpCPairs: null,
MappingPairs: null,

//------------------------------------------------------------------------------
//	Generate cleaning pattern and lookup table from transformation table
//------------------------------------------------------------------------------

tablePairsCleaning: [

	{ m: 'Besten',	f: 'Beste'  },
	{ m: 'Bester',	f: 'Beste'  },
	{ m: 'Dein',	f: 'Deine'  },
	{ m: 'Deinem',	f: 'Deiner' },
	{ m: 'Deinen',	f: 'Deine'  },
	{ m: 'Deines',	f: 'Deiner' },
	{ m: 'Dem',	f: 'Der'    },
	{ m: 'Den',	f: 'Die'    },
	{ m: 'Der',	f: 'Die'    },
	{ m: 'Des',	f: 'Der'    },
	{ m: 'Dessen',	f: 'Deren'  },
	{ m: 'Diesem',	f: 'Dieser' },
	{ m: 'Diesen',	f: 'Diese'  },
	{ m: 'Dieser',	f: 'Diese'  },
	{ m: 'Dieses',	f: 'Dieser' },
	{ m: 'Ein',	f: 'Eine'   },
	{ m: 'Eine',	f: 'Einer'  },
	{ m: 'Einen',	f: 'Eine'   },
	{ m: 'Eines',	f: 'Einer'  },
	{ m: 'Er',	f: 'Sie'    },
	{ m: 'Ihm',	f: 'Ihr'    },
	{ m: 'Ihn',	f: 'Sie'    },
	{ m: 'Jedem',	f: 'Jede'   },
	{ m: 'Jedem',	f: 'Jeder'  },
	{ m: 'Jeden',	f: 'Jede'   },
	{ m: 'Jeder',	f: 'Jede'   },
	{ m: 'Jeder',	f: 'Jedes'  },
	{ m: 'Jedes',	f: 'Jeder'  },
	{ m: 'Kein',	f: 'Keine'  },
	{ m: 'Keines',	f: 'Keiner' },
	{ m: 'Keinem',	f: 'Keiner' },
	{ m: 'Keinen',	f: 'Keine'  },
	{ m: 'Lieben',	f: 'Liebe'  },
	{ m: 'Lieber',	f: 'Liebe'  },
	{ m: 'Man',	f: 'Frau'   },
	{ m: 'Mann',	f: 'Frau',	t: 'Man' },
	{ m: 'Mein',	f: 'Meine'  },
	{ m: 'Meinem',	f: 'Meiner' },
	{ m: 'Meinen',	f: 'Meine'  },
	{ m: 'Meines',	f: 'Meiner' },
	{ m: 'Sein',	f: 'Ihr'    },
	{ m: 'Sein',	f: 'Seine'  },
	{ m: 'Seine',	f: 'Ihre'   },
	{ m: 'Seinem',	f: 'Ihrem'  },
	{ m: 'Seinem',	f: 'Seiner' },
	{ m: 'Seinen',	f: 'Ihren'  },
	{ m: 'Seinen',	f: 'Seine'  },
	{ m: 'Seiner',	f: 'Ihrer'  },
	{ m: 'Seines',	f: 'Ihres'  },
	{ m: 'Seines',	f: 'Seiner' },
	{ m: 'Zum',	f: 'Zur'    }
],

preparePairsCleaning: function() {

	//----------------------------------------------------------------------
	//	caseSensitive pairs
	//----------------------------------------------------------------------

	var wordList = [];
	var mappings = {};

	for( var i = 0 ; i < this.tablePairsCleaning.length ; ++i ) {

		var entry = this.tablePairsCleaning[ i ];

		//--------------------------------------------------------------
		//	uppercase / lowercase
		//--------------------------------------------------------------

		var tc_m = entry.m;
		var tc_f = entry.f;
		var tc_t = entry.t || entry.m;

		var lc_m = tc_m.toLowerCase();
		var lc_f = tc_f.toLowerCase();
		var lc_t = tc_t.toLowerCase();

		//--------------------------------------------------------------
		//	male/female words (pattern ignores case)
		//--------------------------------------------------------------

		wordList.push( lc_m );
		wordList.push( lc_f );

		//--------------------------------------------------------------
		//	uppercase mappings
		//--------------------------------------------------------------

		mappings[ tc_m + '|' + tc_f ] = tc_t;	// Der + Die → Der
		mappings[ tc_f + '|' + tc_m ] = tc_t;	// Die + Der → Der

		//--------------------------------------------------------------
		//	mixed mappings
		//--------------------------------------------------------------

		mappings[ tc_m + '|' + lc_f ] = tc_t;	// Der + die → Der
		mappings[ tc_f + '|' + lc_m ] = tc_t;	// Die + der → Der

		//--------------------------------------------------------------
		//	lowercase mappings
		//--------------------------------------------------------------

		mappings[ lc_m + '|' + lc_f ] = lc_t;	// der + die → der
		mappings[ lc_f + '|' + lc_m ] = lc_t;	// die + der → der
	}

	//----------------------------------------------------------------------
	//	create regexp and lookup table
	//----------------------------------------------------------------------

	var words = '(' + this.uniq( wordList ).sort().join( '|' ) + ')';
	var joiner = '([^\\P{P},]|\\s+oder\\s+|\\sbzw\\.?\\s)';

	this.RegExpCPairs = new RegExp( words + joiner + words, 'ui' );

	this.RegExpPairs = new RegExp( '(?<![\\p{L}\\p{N}\xad])' +
		words + joiner + words + '(?![\\p{L}\\p{N}\xad])', 'ugi' );

	this.MappingPairs = mappings;
},

//------------------------------------------------------------------------------
//
//
//	Pattern:	jede:r, jede/r, ...
//
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	Cleaning pattern and lookup table
//------------------------------------------------------------------------------

RegExpPairs2:  null,
RegExpCPairs2: null,
MappingPairs2: null,

//------------------------------------------------------------------------------
//	Generate cleaning pattern and lookup table from transformation table
//------------------------------------------------------------------------------

preparePairs2Cleaning: function() {

	//----------------------------------------------------------------------
	//	searchpattern and lookup table
	//----------------------------------------------------------------------

	var wordList = [];
	var mappings = {};

	//----------------------------------------------------------------------
	//	jede/r → jeder
	//	jed:r → jeder
	//	Dein/e
	//----------------------------------------------------------------------

	var JOINER = '[*\\p{Pc}\\p{Pd}\\p{Po}]';

	for( var i = 0 ; i < this.tablePairsCleaning.length ; ++i ) {

		//--------------------------------------------------------------
		//	{ m: 'Dein', f: 'Deine'  },
		//--------------------------------------------------------------

		var entry = this.tablePairsCleaning[ i ];

		//--------------------------------------------------------------
		//	check length
		//--------------------------------------------------------------

		var tc_m = entry.m;
		var tc_f = entry.f;

		if( tc_m == tc_f ) { continue; }	// cannot happen

		var mlen = tc_m.length;
		var flen = tc_f.length;
		var clen = this.commonLength( tc_m, tc_f );

		if( clen <      2 ) { continue; }	// min common length
		if( mlen > clen+1 ) { continue; }	// m suffix too large
		if( flen > clen+1 ) { continue; }	// f suffix too large

		//--------------------------------------------------------------
		//	prepare substrings
		//--------------------------------------------------------------

		var tc_c = tc_m.substr( 0, clen );	// common string
		var lc_c = tc_c.toLowerCase();		// common string lc

		var tc_x1 = tc_c.substr( 0, clen-1 );	// tc_c = tc_x1 + tc_x2
		var tc_x2 = tc_c.substr( clen-1    );
		var lc_x1 = tc_x1.toLowerCase();	// lc_c = lc_x1 + tc_x2

		var lc_m  = tc_m.toLowerCase();		// lowercase result

		//--------------------------------------------------------------
		//	zB. Akkusativ: eine Frau, einen Mann: eine/n → einen
		//--------------------------------------------------------------

		if( flen == clen ) {

			var xm = tc_m.substr( clen );		// "n"

			//------------------------------------------------------
			//	Jede/n → Jede:n → Jeden
			//	jede/n → jede:n → jeden
			//------------------------------------------------------

			wordList.push( tc_c + JOINER + xm );
			mappings[      tc_c +   '*'  + xm ] = tc_m;

			wordList.push( lc_c + JOINER + xm );
			mappings[      lc_c +   '*'  + xm ] = lc_m;

			//------------------------------------------------------
			//	JedeN → JedeN → Jeden
			//	jedeN → jedeN → jeden
			//------------------------------------------------------

			var uc_xm = xm.toUpperCase();

			wordList.push( tc_c + uc_xm );
			mappings[      tc_c + uc_xm ] = tc_m;

			wordList.push( lc_c + uc_xm );
			mappings[      lc_c + uc_xm ] = lc_m;

			//------------------------------------------------------
			//	Jede(n)	→ Jede:n: → Jeden
			//	jede(n)	→ jede:n: → jeden
			//------------------------------------------------------

			wordList.push( tc_c + '(' + xm + ')' );
			mappings[      tc_c + '*' + xm + '*' ] = tc_m;

			wordList.push( lc_c + '(' + xm + ')' );
			mappings[      lc_c + '*' + xm + '*' ] = lc_m;

			//------------------------------------------------------
			//	Jed:n → Jed:n → Jeden
			//	jed:n → jed:n → jeden
			//------------------------------------------------------

			if( tc_x2 === 'e' ) {

				wordList.push( tc_x1 + ':' + xm );
				mappings[      tc_x1 + '*' + xm ] = tc_m;

				wordList.push( lc_x1 + ':' + xm );
				mappings[      lc_x1 + '*' + xm ] = lc_m;
			}
		}

		//--------------------------------------------------------------
		//	zB. Nominativ: ein Mann, eine Frau: ein/e → ein
		//--------------------------------------------------------------

		if( mlen == clen ) {

			var xf = tc_f.substr( clen );

			//------------------------------------------------------
			//	Ein/e → Ein:e → Ein
			//	ein/e → ein:e → ein
			//------------------------------------------------------

			wordList.push( tc_c + JOINER + xf );
			mappings[      tc_c +   '*'  + xf ] = tc_m;

			wordList.push( lc_c + JOINER + xf );
			mappings[      lc_c +   '*'  + xf ] = lc_m;

			//------------------------------------------------------
			//	EinE → EinE → Ein
			//	einE → einE → ein
			//------------------------------------------------------

			var uc_xf = xf.toUpperCase();

			wordList.push( tc_c + uc_xf );
			mappings[      tc_c + uc_xf ] = tc_m;

			wordList.push( lc_c + uc_xf );
			mappings[      lc_c + uc_xf ] = lc_m;

			//------------------------------------------------------
			//	Ein(e) → Ein:e :→ Ein
			//	ein(e) → ein:e: → ein
			//------------------------------------------------------

			wordList.push( tc_c + '(' + xf + ')' );
			mappings[      tc_c + '*' + xf + '*' ] = tc_m;

			wordList.push( lc_c + '(' + xf + ')' );
			mappings[      lc_c + '*' + xf + '*' ] = lc_m;

			//------------------------------------------------------
			//	jed:n
			//------------------------------------------------------

			if( tc_x2 === 'e' ) {

				wordList.push( tc_x1 + ':' + xf );
				mappings[      tc_x1 + '*' + xf ] = tc_m;

				wordList.push( lc_x1 + ':' + xf );
				mappings[      lc_x1 + '*' + xf ] = lc_m;
			}
		}

		//--------------------------------------------------------------
		//	zB. Genitiv: eines Mannes, einer Frau: einer/s → eines
		//--------------------------------------------------------------

		if( mlen === flen ) {

			xm = tc_m.substr( clen );
			xf = tc_f.substr( clen );

			//------------------------------------------------------
			//	Eines/r → Eines:r → Eines
			//	eines/r → eines:r → eines
			//------------------------------------------------------

			wordList.push( tc_c + xm + JOINER + xf );
			mappings[      tc_c + xm +   '*'  + xf ] = tc_m;

			wordList.push( lc_c + xm + JOINER + xf );
			mappings[      lc_c + xm +   '*'  + xf ] = lc_m;

			//------------------------------------------------------
			//	Einer/s → Einer:s → Eines
			//	einer/s → einer:s → eines
			//------------------------------------------------------

			wordList.push( tc_c + xf + JOINER + xm );
			mappings[      tc_c + xf +   '*'  + xm ] = tc_m;

			wordList.push( lc_c + xf + JOINER + xm );
			mappings[      lc_c + xf +   '*'  + xm ] = lc_m;
		}
	}

	//----------------------------------------------------------------------
	//	create regexp and lookup table
	//----------------------------------------------------------------------

	this.RegExpCPairs2 = new RegExp(
		'(' +
			this.uniq( wordList ).sort().reverse().join( '|' ) +
		')' +
		'', 'u' );

	this.RegExpPairs2 = new RegExp(
		// word boundary
		'(?<!' +
			'\\p{L}' +
		')' +
		'(' +
			this.uniq( wordList ).sort().reverse().join( '|' ) +
		')' +
		// word boundary
		'(?!' +
			'\\p{L}' +
		')' +
		'', 'ug' );

	this.MappingPairs2 = mappings;
},

//------------------------------------------------------------------------------
//
//
//	Pattern:	"neue*n" → "neuen"
//			"organisatorischen*n" → "organisatorische*n"
//
//
//------------------------------------------------------------------------------
//	Cleaning pattern and lookup table
//------------------------------------------------------------------------------

RegExpCEndings:  new RegExp(
	'(' +
		'\\p{L}*' +
	')' +
	'(' +
		'e[*][nr]' +
	')' +
	'', 'u' ),

RegExpEndings:  new RegExp(
	// word boundary
	'(?<!' +
		'[\\p{L}\\p{N}]' +
	')' +
	'(' +
		'\\p{L}+' +
	')' +
	'(' +
		'e[*][nr]' +
	')' +
	// word boundary
	'(?!' +
		'[\\p{L}\\p{N}]' +
	')' +
	'', 'ug' ),

MappingEndings: {
	'e*n': 'en',
	'e*r': 'er'
},

//==============================================================================
//
//	S I M P L E   W O R D   R E P L A C E M E N T S
//
//==============================================================================

//------------------------------------------------------------------------------
//
//
//	Pattern:	word → replacement (niemensch → niemand)
//
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	Cleaning pattern and lookup table
//------------------------------------------------------------------------------

RegExpCWords: null,
RegExpWords:  null,
MappingWords: null,

RegExpCWords2: null,
RegExpWords2:  null,
MappingWords2: null,

//------------------------------------------------------------------------------
//	Generate cleaning pattern and lookup table from transformation table
//------------------------------------------------------------------------------

// no prefix
tableWordsLcCleaning: [

	{ g: 'Elter',		c: 'Elternteil' },
	{ g: 'Jefrau',		c: 'Jemand'  },
	{ g: 'Jefraud',		c: 'Jemand'  },
	{ g: 'Jemensch',	c: 'Jemand'  },
	{ g: 'Jemenschd',	c: 'Jemand'  },
	{ g: 'Niefrau',		c: 'Niemand' },
	{ g: 'Niefraud',	c: 'Niemand' },
	{ g: 'Niemensch',	c: 'Niemand' },
	{ g: 'Niemenschd',	c: 'Niemand' }
],

// with prefix ok
tableWordsUcCleaning: [

	{ g: 'Gästin',		c: 'Gast'    },
	{ g: 'Gästinnen',	c: 'Gäste'   },
	{ g: 'Männin',		c: 'Frau'    },
	{ g: 'Männinnen',	c: 'Frauen'  },
	{ g: 'Menschin',	c: 'Frau'    },
	{ g: 'Menschinnen',	c: 'Frauen'  },
	{ g: 'Rom*nja',		c: 'Roma'    },
	{ g: 'Sinti*ze',	c: 'Sinti'   },
	{ g: 'Sinti*zze',	c: 'Sinti'   },
	{ g: 'Vorständin',	c: 'Vorstand'}
],

prepareWordsCleaning: function() {

	//----------------------------------------------------------------------
	//	searchpattern and lookup table
	//----------------------------------------------------------------------

	var wordList = [];
	var mappings = {};

	var wordList2 = [];
	var mappings2 = {};

	//----------------------------------------------------------------------
	//	jefrau → jemand
	//----------------------------------------------------------------------

	for( var i = 0 ; i < this.tableWordsLcCleaning.length ; ++i ) {

		var entry = this.tableWordsLcCleaning[ i ];

		//--------------------------------------------------------------
		//	titlecase gendered → cleaned
		//--------------------------------------------------------------

		var tc_g = entry.g;
		var tc_c = entry.c;

		wordList.push( tc_g );
		mappings[ tc_g ] = tc_c;

		//--------------------------------------------------------------
		//	lowercase gendered → cleaned
		//--------------------------------------------------------------

		var lc_g = tc_g.toLowerCase();
		var lc_c = tc_c.toLowerCase();

		wordList.push( lc_g );
		mappings[ lc_g ] = lc_c;
	}

	//----------------------------------------------------------------------
	//	Gästin → Gast
	//----------------------------------------------------------------------

	for( i = 0 ; i < this.tableWordsUcCleaning.length ; ++i ) {

		entry = this.tableWordsUcCleaning[ i ];

		//--------------------------------------------------------------
		//	titlecase gendered → cleaned
		//--------------------------------------------------------------

		tc_g = entry.g;
		tc_c = entry.c;

		wordList.push( tc_g );
		mappings[ tc_g ] = tc_c;

		//--------------------------------------------------------------
		//	titlecase gendered → cleaned
		//--------------------------------------------------------------

		lc_g = tc_g.toLowerCase();
		lc_c = tc_c.toLowerCase();

		wordList2.push( lc_g );
		mappings2[ lc_g ] = lc_c;
	}

	//----------------------------------------------------------------------
	//	create regexp and lookup table
	//----------------------------------------------------------------------

	this.RegExpCWords = new RegExp(
		'(' +
			wordList.sort().join( '|' ).replace( /[*]/g, '[*]' ) +
		')' +
		'', 'u' );

	this.RegExpWords = new RegExp(
		// word boundary
		'(?<!' + '\\p{L}' + ')' +
		'(' +
			wordList.sort().join( '|' ).replace( /[*]/g, '[*]' ) +
		')' +
		// word boundary
		'(?!' + '\\p{L}' + ')' +
		'', 'ug' );

	this.MappingWords = mappings;

	this.RegExpCWords2 = new RegExp(
		'(' +
			'\\p{Lu}\\p{Ll}*' +
		')' +
		'(' +
			wordList2.sort().join( '|' ).replace( /[*]/g, '[*]' ) +
		')' +
		'', 'u' );

	this.RegExpWords2 = new RegExp(
		// word boundary
		'(?<!' + '\\p{L}' + ')' +
		'(' +
			'\\p{Lu}\\p{Ll}*' +
		')' +
		'(' +
			wordList2.sort().join( '|' ).replace( /[*]/g, '[*]' ) +
		')' +
		// word boundary
		'(?!' + '\\p{L}' + ')' +
		'', 'ug' );

	this.MappingWords2 = mappings2;
},

//------------------------------------------------------------------------------
//
//
//	Pattern:	( m / w / d )
//
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	Case insensitive pattern
//------------------------------------------------------------------------------

RegExpMWD: new RegExp(
	'(\\p{Ps})' +
		'\\s*' + '(' + '[mwd]' + ')' + '\\s*' +
		'/' +
		'\\s*' + '(' + '[mwd]' + ')' + '\\s*' +
		'/' +
		'\\s*' + '(' + '[mwd]' + ')' + '\\s*' +
	'(\\p{Pe})', 'ugi' ),

//==============================================================================
//
//	S I M P L E   W O R D   R E P L A C E M E N T S
//
//==============================================================================

//------------------------------------------------------------------------------
//
//
//	Clowning and Cleaning filter methods
//
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	unmodifyingy coarse filter to avoid unnecessary processing 
//------------------------------------------------------------------------------

coarseFilter: function( textContent, settings ) {

	var res = 0;

	//----------------------------------------------------------------------
	//	Binnen-I
	//----------------------------------------------------------------------

	if( this.RegExpCInCS.    test( textContent )) { res |= 0x0001; }
	if( this.RegExpCInCI.    test( textContent )) { res |= 0x0002; }
	if( this.RegExpCInnenCS. test( textContent )) { res |= 0x0004; }
	if( this.RegExpCInnenCI. test( textContent )) { res |= 0x0008; }

	//----------------------------------------------------------------------
	//	Present participle
	//----------------------------------------------------------------------

	if( this.RegExpCParticiple.test( textContent )) { res |= 0x0010; }

	//----------------------------------------------------------------------
	//	Duplicate forms
	//----------------------------------------------------------------------

	if( this.RegExpCDuplicates.test( textContent )) { res |= 0x0100; }
	if( this.RegExpCPairs.     test( textContent )) { res |= 0x0200; }
	if( this.RegExpCPairs2.    test( textContent )) { res |= 0x0400; }
	if( this.RegExpCEndings.   test( textContent )) { res |= 0x0800; }

	//----------------------------------------------------------------------
	//	Simple replacements
	//----------------------------------------------------------------------

	if( this.RegExpCWords. test( textContent )) { res |= 0x1000; }
	if( this.RegExpCWords2.test( textContent )) { res |= 0x1000; }
	if( this.RegExpMWD.    test( textContent )) { res |= 0x2000; }

	return res;
},

//------------------------------------------------------------------------------
//	fine filter modifies node data
//------------------------------------------------------------------------------

BOC: '\x01',
BOT: '\x02',
EOC: '\x03',

RegExpDativPluralPrefix: new RegExp(
	'(?<!' +
		'\\p{L}' +
	')' +
	'(' +
		'(?:' +
			'[Aa]b|[Aa]ußer|[Bb]ei|[Dd]ank' +
		'|' +
			'(?<!,\\s*)[Dd]en' +
		'|' +
			'[Gg]egenüber|[Hh]inter|[Ll]aut|[Nn]ach|[Nn]eben|[Mm]it|[Uu]nter|[Vv]on|[Zz]u' +
		'|' +
			'hilft' +
		')' +
		'\\x20' +
	')' +
	'(' +
		'(?:' +
			'(?!' + 'die\\x20' + ')' +
			'(?:' +
				'\\p{Ll}+' +			// von neuen
			'|' +
				'\\p{L}+' + '-' +		// von X- und Y-Arbeitern
			'|' +
				'\\p{N}+(?:[,.]\\p{N}+)*' +	// von 2.300
			')' +
			'\\x20' +
		'){0,2}' +
	')' +
	'(?:' +
		'\\p{L}+-' +		// EU-Parlamentarier
	')*' +
	'$', 'u' ),

RegExpDerOderDiePrefix: new RegExp(
	'(?<!' +
		'\\p{L}' +
	')' +
	'(' +
		'der\x20oder\x20die' +
	'|' +
		'die\x20oder\x20der' +
	')' +
	'\x20' +
	'$', 'ui' ),

processNode: function( mode, node, settings ) {

	var s = node.data;

	//----------------------------------------------------------------------
	//	Clown
	//----------------------------------------------------------------------

	var clown = settings.doInsertClowns ? settings.clown : '';

	//----------------------------------------------------------------------
	//	Change marker
	//----------------------------------------------------------------------

	var doMark  = settings.doMarkChanges && 
				node.parentNode.nodeName != 'TITLE';

	var boc     = doMark ? this.BOC : '';
	var bot     = doMark ? this.BOT : '';
	var eoc     = doMark ? this.EOC : '';

	//----------------------------------------------------------------------
	//	Global values
	//----------------------------------------------------------------------

	var closure = this;
	var count   = 0;

	//======================================================================
	//	Binnen I
	//======================================================================

	//----------------------------------------------------------------------
	//	Replace "*In"
	//----------------------------------------------------------------------

	function replaceIn( all, p1, p2 ) {

		//--------------------------------------------------------------
		//	skip short words
		//--------------------------------------------------------------

		if( p1.length < 3 ) return all;

		//--------------------------------------------------------------
		//	skip english words resembling gender language
		//--------------------------------------------------------------

		switch( p1.toLowerCase().replace( /\xad/g, '' )) {

		case 'all':		// all-in-one
		case 'built':		// Build-in
		case 'call':		// Call-in-show
		case 'check':		// Check-in
		case 'drive':		// Drive-in
		case 'drop':		// Drop-in-replacement
		case 'editor':		// Editor-in-chief
		case 'gov':		// Domain: gov.in
		case 'hand':
		case 'just':		// just-in-time
		case 'kiss':
		case 'linked':		// LinkedIn
		case 'lock':
		case 'locked':
		case 'log':
		case 'made':		// made-in
		case 'man':		// man-in-the-middle
		case 'opt':		// opt-in
		case 'plug':		// PlugIn-Hybrid
		case 'second':		// second-in-charge
		case 'sign':		// sign-in
		case 'sit':
		case 'study':
			return all;
		default:
			break;
		}

		//--------------------------------------------------------------
		//	replace
		//--------------------------------------------------------------

		var repl = all;

		if( settings.doCleaning ) {

			var match = p1.replace( /\xad/g, '' ).
					match( closure.RegExpInCleaning );
			if( !match ) { return all; }

			var trans = closure.MappingInCleaning[ match[ 2 ] ];

			if( settings.doLogging ) {

				console.debug( '[ ' + match[ 1 ] + ' | ' +
					match[ 2 ] + ' ]→ ' + trans );
			}

			if( !trans ) { return all; }

			if( p1 == p1.toUpperCase() ) trans = trans.toUpperCase();
			repl = match[ 1 ] + trans + clown;
		} else {

			repl = p1 + clown + p2;
		}

		//--------------------------------------------------------------
		//	log
		//--------------------------------------------------------------

		if( settings.doLogging ) {

			console.info( '“' + all + '” → “' + repl + '”' );
		}

		//--------------------------------------------------------------
		//	return
		//--------------------------------------------------------------

		++count;
		return boc + repl + ( bot? bot + all : '' ) + eoc;
	}

	//----------------------------------------------------------------------
	//	{Letters}In
	//----------------------------------------------------------------------

	if( mode & 0x0001 ) { s = s.replace( this.RegExpInCS, replaceIn ); }
	if( mode & 0x0002 ) { s = s.replace( this.RegExpInCI, replaceIn ); }

	//----------------------------------------------------------------------
	//	Replace p1 = <word>, p2 = "innen", p3 = <compound>
	//----------------------------------------------------------------------

	function replaceInnen( all, p1, p2, p3, pos, full ) {

		//--------------------------------------------------------------
		//	skip short words
		//--------------------------------------------------------------

		if( p1.length < 3 ) return all;

		//--------------------------------------------------------------
		//	replace
		//--------------------------------------------------------------

		var repl = all;

		if( settings.doCleaning ) {

			//------------------------------------------------------
			//	map match
			//------------------------------------------------------

			var match = p1.replace( /\xad/g, '' ).
					match( closure.RegExpInnenCleaning );
			if( !match ) { return all; }

			//------------------------------------------------------
			//	match[ 1 ] = word stem
			//	match[ 2 ] = word ending
			//------------------------------------------------------

			var trans = closure.MappingInnenCleaning[ match[ 2 ] ];

			if( settings.doLogging ) {

				console.debug( '[ ' + match[ 1 ] + ' | ' +
					match[ 2 ] + ' ]→ ' + trans );
			}

			if( !trans ) { return all; }

			//------------------------------------------------------
			//	Append "n" to Dativ plural not ending in "n"
			//------------------------------------------------------

			if( trans.substr( -1 ) != 'n' && !p3 &&
		closure.RegExpDativPluralPrefix.test( full.substr( 0, pos ))) {

				trans += 'n';
			}

			//------------------------------------------------------
			//	UPPERCASE
			//------------------------------------------------------

			if( p1 == p1.toUpperCase() ) trans = trans.toUpperCase();
			repl = match[ 1 ] + trans + p3 + clown;
		} else {

			repl = p1 + clown + p2 + p3;
		}

		//--------------------------------------------------------------
		//	log
		//--------------------------------------------------------------

		if( settings.doLogging ) {

			console.info( '“' + all + '” → “' + repl + '”' );
		}

		//--------------------------------------------------------------
		//	return
		//--------------------------------------------------------------

		++count;
		return boc + repl + ( bot? bot + all : '' ) + eoc;
	}

	//----------------------------------------------------------------------
	//	{Letters}Innen
	//----------------------------------------------------------------------

	if( mode & 0x0004 ) { s = s.replace( this.RegExpInnenCS, replaceInnen ); }
	if( mode & 0x0008 ) { s = s.replace( this.RegExpInnenCI, replaceInnen ); }

	//======================================================================
	//	Participle
	//======================================================================

	if( mode & 0x0010 ) {

		s = s.replace( this.RegExpParticiple,
				function( all, p1, p2, p3, p4, pos, full ) {

			//------------------------------------------------------
			// TODO BERÜCKSICHTIGE DEN SUFFIX (e,en,er,e/r) TODO
			//------------------------------------------------------

			//------------------------------------------------------
			//	replace
			//------------------------------------------------------

			var index = p1.substr( p1.length-1 ) + p2 + p3;
			var trans = closure.MappingParticiple[ index ];

			/*
			console.debug( '‹' + p1 + '|' + p2 + '|' + p3 + '|' +
				p4 + '›' + ' → ' + index + ' → ' + trans );
			*/

			if( !trans ) { return all; }

			var prefix = p1.substr( 0, p1.length-1 ) + trans;
			var suffix =
				// Forsch|er → Forscher|
					p3 != 'ierend' ?
						'' :
				// Studierender oder Studierende → Student
					p2.indexOf( '\x20oder\x20' ) > 0 ?
						'' :
				// der oder die Studierende
		closure.RegExpDerOderDiePrefix.test( full.substr( 0, pos )) ?
						'' :
				// Studierende
					'en';

			//------------------------------------------------------
			//	Append "n" to Dativ plural not ending in "n"
			//------------------------------------------------------

			if( suffix.substr( -1 ) != 'n' &&
		closure.RegExpDativPluralPrefix.test( full.substr( 0, pos ))) {

				suffix += 'n';
			}

			var repl = all;

			if( settings.doCleaning ) {

				repl = prefix + suffix + clown;
			} else {

				repl = all + clown;
			}

			//------------------------------------------------------
			//	log
			//------------------------------------------------------

			if( settings.doLogging ) {

				console.info( '“' + all + '” → “' + repl + '”' );
			}

			//------------------------------------------------------
			//	return
			//------------------------------------------------------

			++count;
			return boc + repl + ( bot? bot + all : '' ) + eoc;
		});
	}

	//======================================================================
	//	Duplicate forms
	//======================================================================

	//----------------------------------------------------------------------
	//	"Beamtinnen und Beamte"
	//----------------------------------------------------------------------
	//	(Beamt)innen und (Beamte) | (Beamt(e)) und (Beamt)innen
	//----------------------------------------------------------------------

	if( mode & 0x0100 ) {

		s = s.replace( this.RegExpDuplicates, function( all, p1, p2, p3 ) {

			var trans = p2 || p3;

			//------------------------------------------------------
			//	replace
			//------------------------------------------------------

			var repl = all;

			if( settings.doCleaning ) {

				repl = trans + clown;
			} else {

				repl = all + clown;
			}

			//------------------------------------------------------
			//	log
			//------------------------------------------------------

			if( settings.doLogging ) {

				console.info( '“' + all + '” → “' + repl + '”' );
			}

			//------------------------------------------------------
			//	return
			//------------------------------------------------------

			++count;
			return boc + repl + ( bot? bot + all : '' ) + eoc;
		});
	}

	//----------------------------------------------------------------------
	//	"Der / Die" → "Der"; "Der bzw. die" → "Der" 
	//----------------------------------------------------------------------

	if( mode & 0x0200 ) {

		s = s.replace( this.RegExpPairs, function( all, p1, p2, p3 ) {

			var trans = closure.MappingPairs[ p1 + '|' + p3 ];
			if( !trans ) { return all; }

			//------------------------------------------------------
			//	replace
			//------------------------------------------------------

			var repl = all;

			if( settings.doCleaning ) {

				repl = trans + clown;
			} else {

				repl = p1 + clown + p2 + p3;
			}

			//------------------------------------------------------
			//	log
			//------------------------------------------------------

			if( settings.doLogging ) {

				console.info( '“' + all + '” → “' + repl + '”' );
			}

			//------------------------------------------------------
			//	return
			//------------------------------------------------------

			++count;
			return boc + repl + ( bot? bot + all : '' ) + eoc;
		});
	}

	//----------------------------------------------------------------------
	//	"jede*r" → "jeder"; "jede(r)" → "jeder"; "jedes/r" → "jedes"
	//----------------------------------------------------------------------

	if( mode & 0x0400 ) {

		s = s.replace( this.RegExpPairs2, function( all, p1 ) {

			var p1x = p1.replace( closure.RegExpNoLetters, '*' );

			var trans = closure.MappingPairs2[ p1x ];
			if( !trans ) { return all; }

			//------------------------------------------------------
			//	replace
			//------------------------------------------------------

			var repl = all;

			if( settings.doCleaning ) {

				repl = trans + clown;
			} else {

				repl = all + clown;
			}

			//------------------------------------------------------
			//	log
			//------------------------------------------------------

			if( settings.doLogging ) {

				console.info( '“' + all + '” → “' + repl + '”' );
			}

			//------------------------------------------------------
			//	return
			//------------------------------------------------------

			++count;
			return boc + repl + ( bot? bot + all : '' ) + eoc;
		});
	}

	//----------------------------------------------------------------------
	//	"neue*n" → "neuen"
	//	"organisatorischen*n" → "organisatorischen"
	//----------------------------------------------------------------------

	if( mode & 0x0800 ) {

		s = s.replace( this.RegExpEndings, function( all, stem, ending ) {

			var trans = closure.MappingEndings[ ending ];
			if( !trans ) return all;

			if( closure.MappingPairs2[ all ]) return all;

			//------------------------------------------------------
			//	replace
			//------------------------------------------------------

			var repl = all;

			if( settings.doCleaning ) {

				repl = stem + trans + clown;
			} else {

				repl = all + clown;
			}

			//------------------------------------------------------
			//	log
			//------------------------------------------------------

			if( settings.doLogging ) {

				console.info( '“' + all + '” → “' + repl + '”' );
			}

			//------------------------------------------------------
			//	return
			//------------------------------------------------------

			++count;
			return boc + repl + ( bot? bot + all : '' ) + eoc;
		});
	}

	//======================================================================
	//	Simple word replacements
	//======================================================================

	//----------------------------------------------------------------------
	//	niefraud → niemand
	//----------------------------------------------------------------------

	if( mode & 0x1000 ) {

		s = s.replace( this.RegExpWords, function( all, p1 ) {

			var trans = closure.MappingWords[ p1 ];
			if( !trans ) { return all; }

			//------------------------------------------------------
			//	replace
			//------------------------------------------------------

			var repl = all;

			if( settings.doCleaning ) {

				repl = trans + clown;
			} else {

				repl = all + clown;
			}

			//------------------------------------------------------
			//	log
			//------------------------------------------------------

			if( settings.doLogging ) {

				console.info( '“' + all + '” → “' + repl + '”' );
			}

			//------------------------------------------------------
			//	return
			//------------------------------------------------------

			++count;
			return boc + repl + ( bot? bot + all : '' ) + eoc;
		});

		s = s.replace( this.RegExpWords2, function( all, p1, p2 ) {

			var trans = closure.MappingWords2[ p2 ];
			if( !trans ) { return all; }

			//------------------------------------------------------
			//	replace
			//------------------------------------------------------

			var repl = all;

			if( settings.doCleaning ) {

				repl = p1 + trans + clown;
			} else {

				repl = all + clown;
			}

			//------------------------------------------------------
			//	log
			//------------------------------------------------------

			if( settings.doLogging ) {

				console.info( '“' + all + '” → “' + repl + '”' );
			}

			//------------------------------------------------------
			//	return
			//------------------------------------------------------

			++count;
			return boc + repl + ( bot? bot + all : '' ) + eoc;
		});
	}

	//----------------------------------------------------------------------
	//	( m / w / d )   [all cases and permutations]
	//----------------------------------------------------------------------

	if( mode & 0x2000 ) {

		s = s.replace( this.RegExpMWD, function( all, p1, p2, p3, p4, p5 ) {

			if( [
				p2.toLowerCase(),
				p3.toLowerCase(),
				p4.toLowerCase()
			].sort().join( '' ) !== 'dmw' ) {

				return all;
			}

			//------------------------------------------------------
			//	replace
			//------------------------------------------------------

			var repl = all;

			if( settings.doCleaning ) {

				repl = clown;

			} else if( clown ) {

				repl = p1 +
					p2 + '/' + p3 + '/' + p4 + '/' + clown +
					p5;
			}

			//------------------------------------------------------
			//	log
			//------------------------------------------------------

			if( settings.doLogging ) {

				console.info( '“' + all + '” → “' + repl + '”' );
			}

			//------------------------------------------------------
			//	return
			//------------------------------------------------------

			++count;
			return boc + repl + ( bot? bot + all : '' ) + eoc;
		});
	}

	//======================================================================
	//	no changes
	//======================================================================

	if( s === node.data ) return count;

	//----------------------------------------------------------------------
	//	no marks
	//----------------------------------------------------------------------

	if( s.indexOf( this.BOC ) < 0 ) {

		node.data = s;
		return count;
	}

	//----------------------------------------------------------------------
	//	process change marks
	//----------------------------------------------------------------------

	var parentNode  = node.parentNode;
	var segments    = s.split( this.EOC );

	for( var i = 0 ; i < segments.length-1 ; ++i ) {

		var fields = segments[ i ].split( this.BOC );

		//--------------------------------------------------------------
		//	unchanged text
		//--------------------------------------------------------------

		if( fields[ 0 ] ) {

			parentNode.insertBefore(
				document.createTextNode( fields[ 0 ] ), node );
		}

		//--------------------------------------------------------------
		//	replacement
		//--------------------------------------------------------------

		if( fields[ 1 ] ) {

			//------------------------------------------------------
			//	split → replacement + original
			//------------------------------------------------------

			var subs = fields[ 1 ].split( bot );

			//------------------------------------------------------
			//	marked replacement
			//------------------------------------------------------

			var mark = document.createElement(
							settings.markElement );
			mark.className = 'no-genderclown';
			mark.style = settings.markStyle || '';

			//------------------------------------------------------
			//	replacement → mark content
			//	original    → mark title
			//------------------------------------------------------

			mark.appendChild( document.createTextNode( subs[ 0 ]));

			if( subs[ 1 ]) {

				mark.title = subs[ 1 ];
				mark.style.cursor = 'help';
			}

			//------------------------------------------------------
			//	insert mark element into parent element
			//------------------------------------------------------

			parentNode.insertBefore( mark, node );
		}
	}

	node.data = segments[ segments.length-1 ];
	return count;
},

//------------------------------------------------------------------------------
//
//	Auxiliary methods
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	compare strings longest first
//------------------------------------------------------------------------------

byLength: function( a, b ) {

	return b.length - a.length || ( a<b ? -1 : a>b ? +1 : 0 );
},

//------------------------------------------------------------------------------
//	remove double entries from array
//------------------------------------------------------------------------------

uniq: function( list ) {

	var seen = {};

	for( var i = 0 ; i < list.length ; ++i ) {

		var a = list[ i ];

		if( seen[ a ] ) {

			list.splice( i,  1 );
			--i;
		} else {
			seen[ a ] = true;
		}
	}

	return list;
},

//------------------------------------------------------------------------------
//	lookup common substring (not yet used)
//------------------------------------------------------------------------------

commonLength: function( s1, s2 ) {

	var minLength = Math.min( s1.length, s2.length );

	var len = 0;

	while( len < minLength ) {

		if( s1.charAt( len ) != s2.charAt( len )) { break; }
		++len;
	}

	return len;
},

//------------------------------------------------------------------------------
//
//	initialization
//
//------------------------------------------------------------------------------

init: function() {

	this.prepareInCleaning();
	this.prepareInnenCleaning();
	this.preparePairsCleaning();
	this.preparePairs2Cleaning();
	this.prepareParticipleCleaning();
	this.prepareWordsCleaning();
}

}));

//------------------------------------------------------------------------------
//	$Id: node-modifier.js,v 1.80 2021/07/14 20:36:56 wolf Exp $
//------------------------------------------------------------------------------
