//------------------------------------------------------------------------------
//	$Id: options.js,v 1.10 2021/02/24 02:42:27 wolf Exp $
//------------------------------------------------------------------------------
//	See:	https://www.netzwolf.info/code/addons/genderclown/
//------------------------------------------------------------------------------
//	License: WTF public license (see: http://www.wtfpl.net/)
//------------------------------------------------------------------------------

/*jsl:option explicit*/

'use strict';

document.addEventListener( 'DOMContentLoaded', function() {

	//----------------------------------------------------------------------
	//	Chrome :-(
	//----------------------------------------------------------------------

	if( typeof( browser ) === 'undefined' ) {

		console = chrome.extension.getBackgroundPage().console;
	}

	//----------------------------------------------------------------------
	//	log script startet
	//----------------------------------------------------------------------

	console.debug( 'options.js started @' + new Date().toTimeString() );

	//----------------------------------------------------------------------
	//	translate page
	//----------------------------------------------------------------------

	var i18nElements = document.getElementsByClassName( 'i18n' );

	for( var e = 0 ; e < i18nElements.length ; ++e ) {

		var ele  = i18nElements[ e ];

		var text = chrome.i18n.getMessage( ele.id );
		if( !text ) continue;

		while( ele.firstChild ) ele.removeChild( ele.firstChild );
		ele.appendChild( document.createTextNode( text ));
	}

	//----------------------------------------------------------------------
	//	settings and form
	//----------------------------------------------------------------------

	var settings = {};

	var form = document.forms[ 0 ];

	//----------------------------------------------------------------------
	//	store settings
	//----------------------------------------------------------------------

	function storeSettings() {

		//--------------------------------------------------------------
		//	update settings to form element values
		//--------------------------------------------------------------

		for( var e = 0 ; e < form.elements.length ; ++ e ) {

			var element = form.elements[ e ];

			//------------------------------------------------------
			//	handle different element types
			//------------------------------------------------------

			switch( element.type ) {

			case 'checkbox':
				settings[ element.name ] = element.checked;
				break;

			case 'radio':
				if( !element.checked ) break;
				settings[ element.name ] = element.value;
				break;

			case 'select-one':
				settings[ element.name ] = element.value;
				break;

			case '':
			case 'text':
			case 'textarea':
				settings[ element.name ] = element.value;
				break;

			default:
				break;
			}
		}

		//--------------------------------------------------------------
		//	send settings to background script
		//--------------------------------------------------------------

		var message = {
			'action': 'writeSettings',
			'settings': JSON.stringify( settings )
		};

		chrome.runtime.sendMessage( message );
	}

	//----------------------------------------------------------------------
	//	auxiliary: store list of hostnames to textarea
	//	add rows to textarea if needed
	//----------------------------------------------------------------------

	var InvalidCharsRegExp = new RegExp( '[^\\s\\p{L}\\p{N}\\p{S}*,.-]+', 'gu' );

	function storeDomains( element, value ) {

		var hostnames = value.trim().toLowerCase().
			replace( InvalidCharsRegExp, 'X' ).split( /[,\s]+/ );
		element.value = hostnames.join( '\n' );

		element.setAttribute( 'rows', Math.max(
			hostnames.length+1, element.getAttribute( 'rows' )));
	}

	//----------------------------------------------------------------------
	//	get settings
	//----------------------------------------------------------------------

	chrome.runtime.sendMessage( { action: 'readSettings' }, function( message ) {

		//--------------------------------------------------------------
		//	decode settings
		//--------------------------------------------------------------

    		settings = JSON.parse( message.settings );

		//--------------------------------------------------------------
		//	process form entries
		//--------------------------------------------------------------

		for( var e = 0 ; e < form.elements.length ; ++ e ) {

			var element = form.elements[ e ];

			var value = settings[ element.name || '' ] || '';

			//------------------------------------------------------
			//	handle different element types
			//------------------------------------------------------

			switch( element.type ) {

			case 'button':
				element.onclick = storeSettings;
				break;

			case 'checkbox':
				element.checked = !!value;
				break;

			case 'radio':
				element.checked = element.value === value;
				break;

			case 'select-one':
				element.value = value;
				break;

			case '':
			case 'text':
				element.value = value;
				element.onblur = function() {

					if( this.value.trim() ) return;

					this.value = this.getAttribute(
						'data-default' ) || '';
				};
				break;

			case 'textarea':
				storeDomains( element, value || '' );
				element.onblur = function() {

					storeDomains( this, this.value);
				};
				break;

			default:
				break;
			}
		}
	});

	//----------------------------------------------------------------------
	//	process copy string on click
	//----------------------------------------------------------------------

	var clickcopyElements = document.getElementsByClassName( 'clickcopy' );

	for( var i = 0 ; i < clickcopyElements.length ; ++i ) {

		clickcopyElements[ i ].onclick = function() {

			var targetId      = this.getAttribute( 'data-ref' );
			var targetElement = document.getElementById( targetId );

			targetElement.value = this.textContent;
		};
	}
});

//------------------------------------------------------------------------------
//	$Id: options.js,v 1.10 2021/02/24 02:42:27 wolf Exp $
//------------------------------------------------------------------------------
